import ExchangeManager from "@b/utils/exchange";
import { hasClients, sendMessageToRoute } from "@b/handler/Websocket";
import { logError } from "@b/utils/logger";
import {
  loadBanStatus,
  saveBanStatus,
  handleBanStatus,
  handleExchangeError,
  formatWaitTime,
} from "../utils";

export const metadata = {};

class MarketDataHandler {
  private static instance: MarketDataHandler;
  private accumulatedBuffer: { [key: string]: any } = {};
  private bufferInterval: NodeJS.Timeout | null = null;
  private unblockTime = 0;
  private activeSubscriptions: Set<string> = new Set();
  private exchange: any = null;
  private symbolToStreamKey: { [key: string]: string } = {};

  private constructor() {}

  public static getInstance(): MarketDataHandler {
    if (!MarketDataHandler.instance) {
      MarketDataHandler.instance = new MarketDataHandler();
    }
    return MarketDataHandler.instance;
  }

  private flushBuffer() {
    Object.entries(this.accumulatedBuffer).forEach(([streamKey, data]) => {
      if (Object.keys(data).length > 0) {
        const route = `/api/exchange/market`;
        const payload = { ...data.payload, symbol: data.symbol };
        sendMessageToRoute(route, payload, {
          stream: streamKey, // Do not include the symbol in the stream key for frontend
          data: data.msg,
        });
        delete this.accumulatedBuffer[streamKey];
      }
    });
  }

  private async fetchDataWithRetries(fetchFunction: () => Promise<any>) {
    if (Date.now() < this.unblockTime) {
      throw new Error(
        `Blocked until ${new Date(this.unblockTime).toLocaleString()}`
      );
    }
    return await fetchFunction();
  }

  private async handleSubscription(
    symbol: string,
    type: string,
    interval?: string,
    limit?: number
  ) {
    const frontendStreamKey = `${type}${interval ? `:${interval}` : ""}${
      limit ? `:${limit}` : ""
    }`;
    const internalStreamKey = `${symbol}:${frontendStreamKey}`;

    this.symbolToStreamKey[frontendStreamKey] = symbol;

    const fetchData = {
      ticker: async () => ({
        msg: await this.exchange.watchTicker(symbol),
        payload: { type },
      }),
      ohlcv: async () => ({
        msg: await this.exchange.watchOHLCV(
          symbol,
          interval,
          undefined,
          Number(limit) || 1000
        ),
        payload: { type, interval, limit },
      }),
      trades: async () => ({
        msg: await this.exchange.watchTrades(
          symbol,
          undefined,
          limit ? Number(limit) : 20
        ),
        payload: { type, limit },
      }),
      orderbook: async () => ({
        msg: await this.exchange.watchOrderBook(
          symbol,
          limit ? Number(limit) : 100
        ),
        payload: { type, limit },
      }),
    };

    while (
      this.activeSubscriptions.has(internalStreamKey) &&
      hasClients(`/api/exchange/market`)
    ) {
      try {
        if (Date.now() < this.unblockTime) {
          await new Promise((resolve) => setTimeout(resolve, 1000));
          continue;
        }

        const { msg, payload } = await this.fetchDataWithRetries(() =>
          fetchData[type]()
        );
        this.accumulatedBuffer[frontendStreamKey] = { symbol, msg, payload };

        await new Promise((resolve) => setTimeout(resolve, 250));
      } catch (error) {
        logError("exchange", error, __filename);
        const result = await handleExchangeError(error, ExchangeManager);
        if (typeof result === "number") {
          this.unblockTime = result;
          await saveBanStatus(this.unblockTime);
        } else {
          this.exchange = result;
        }
        await new Promise((resolve) => setTimeout(resolve, 5000));
      }
    }

    this.activeSubscriptions.delete(internalStreamKey);
  }

  public async start(message: any) {
    try {
      this.unblockTime = await loadBanStatus();

      if (typeof message === "string") {
        message = JSON.parse(message);
      }

      const { symbol, type, interval, limit } = message.payload;

      if (!this.bufferInterval) {
        this.bufferInterval = setInterval(this.flushBuffer.bind(this), 500);
      }

      if (!this.exchange) {
        this.exchange = await ExchangeManager.startExchange();
        if (!this.exchange) {
          throw new Error("Failed to start exchange");
        }
      }

      const typeMap = {
        ticker: "watchTicker",
        ohlcv: "watchOHLCV",
        trades: "watchTrades",
        orderbook: "watchOrderBook",
      };

      if (!this.exchange.has[typeMap[type]]) {
        console.info(`Endpoint ${type} is not available`);
        return;
      }

      const internalStreamKey = `${symbol}:${type}${
        interval ? `:${interval}` : ""
      }${limit ? `:${limit}` : ""}`;

      if (!this.activeSubscriptions.has(internalStreamKey)) {
        this.activeSubscriptions.add(internalStreamKey);
        this.handleSubscription(symbol, type, interval, limit);
      }
    } catch (error) {
      logError("exchange", error, __filename);
    }
  }

  public async stop() {
    this.activeSubscriptions.clear();
    if (this.bufferInterval) {
      clearInterval(this.bufferInterval);
      this.bufferInterval = null;
    }
    if (this.exchange) {
      await ExchangeManager.stopExchange();
      this.exchange = null;
    }
  }
}

export default async (data: Handler, message: any) => {
  const handler = MarketDataHandler.getInstance();
  await handler.start(message);
};
